(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_unary.ml,v 1.18 2007/04/26 16:18:32 blanqui Exp $ *)

(** Generate code for the construction function of a ``true'' unary
 constructor.
 A true unary constructor has one argument and this argument is
 not of type list and should not be treated as the variable length
 list of arguments of an associative binary operation.

 The case of an associative binary operator is handled in the module
 [Genr_vary]; we branch to this case when encountering an associative
 operator, since a variable length list of argument cannot exist for a
 true unary operator. *)

open Parsetree;;
open Spec;;
open Code;;
open Relation;;

open Genr_base;;

let genr_fun_1_nonassoc_by_ap cname =

  let rec genr = function
  | Idempotent s -> [Clause (appi (pattern1 cname underscore) "as" x, x)]
  | Involutive -> [Clause (pattern1 cname x, x)]
  | Nilpotent dname -> [Clause (pattern1 cname underscore, Id dname)]
  | Rewrite (p1, eopt, p2) -> [genr_rule p1 eopt p2]
  | Inverse_Neutral (dname, None) -> genr (Inverse_Neutral (dname, Some dname))
  | Inverse_Neutral (dname, Some rname) -> [Clause (Id dname, Id rname)]
  | Distributive_Unary (dname, None) -> genr (Distributive_Unary (dname, Some dname))
  | Distributive_Unary (dname, Some rname) ->
    [Clause (pattern2 dname x y, constr_fun2 rname (constr_fun1 cname y) (constr_fun1 cname x))]
  | Division_by_Absorbent dname ->
    [Clause (Id dname, app1 "failwith" (Id "\"Division by Absorbent\""))]
  | _ -> assert false in
  genr;;

let genr_fun_1_nonassoc fdef arg =
  let cname = fdef.fd_uname in
  let default = constr1 cname (Id arg) in
  match fdef.fd_relations with
  | [] -> default
  | _ ->
    let clauses =
      List.flatten (List.map (genr_fun_1_nonassoc_by_ap cname) fdef.fd_relations)
      @ [Clause (underscore, default)] in
    Match (x, clauses);;

let genr_fun_1 fdef =
  let cname = fdef.fd_uname in

  (* the argument of the function. *)
  let arg = "x" in

  let body =
    if has_associative fdef.fd_relations
    then Genr_vary.genr_fun_1_assoc fdef arg
    else genr_fun_1_nonassoc fdef arg in

  [construction_function_name cname, [arg], body];;
