(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: main.ml,v 1.20 2007/04/27 12:37:02 blanqui Exp $ *)

open Parsetree;;
open Spec;;
open Genr_base;;
open Genr;;
open Genr_sharing;;
open Listutil;;

(* Converts a type declaration from the Caml parsetree
   into a type definition for Moca. *)

let cdef_of_variant (cname, cts, rels, _) =
  { cd_name = cname; cd_arity = List.length cts; cd_relations = rels };;

let tdef_of_ntd (tname, td) =
  match td.ptype_kind with
  | Ptype_variant (l, _) ->
    Some { td_name = tname; td_desc = TypeSum (List.map cdef_of_variant l) }
  | Ptype_record (l, _) ->
    let get_fname (fname, _, _, _) = fname in
    Some { td_name = tname; td_desc = TypeRecord (List.map get_fname l) }
  | Ptype_abstract (rels, _) ->
    (match td.ptype_manifest with
     | Some ty ->
       Some { td_name = tname; td_desc = TypeAbbrev (ty, rels)}
     | _ -> None);;

(* Extracts the type declarations from a structure or raise an exception *)

exception Mod_sigitem_not_allowed;;

let get_ntds si =
  match si.pstr_desc with
  | Pstr_type ntds -> ntds
  | _ -> raise Mod_sigitem_not_allowed;;

(* The functions that check the admissibility of a mlm file
   and generate the corresponding interface and implementation. *)

let check_and_genr sharing ob si =
  let ntds = get_ntds si in
  Check.sigtype ntds;
  if sharing then print_sharing ob ntds;
  let tdefs = filter_map_opt tdef_of_ntd ntds in
  let topexp = topexp_of_tdefs tdefs in
  Code.structure ob topexp;;

(* The current mlm source file. *)
let input_file_name = ref "-";;
let get_input_file_name () = !input_file_name;;
let set_input_file_name fname =
  if !input_file_name = "-" then input_file_name := fname
  else failwith "Input file name is already set.";;

(* The current output file name (the name of the generated module). *)
let output_module_name = ref "-";;
let get_output_module_name () = !output_module_name;;
let set_output_module_name fname =
  if !output_module_name = "-" then output_module_name := fname
  else failwith "Output module name is already set.";;

let output_module_implementation_name = ref "-";;
let get_output_module_implementation_name () =
  !output_module_implementation_name;;
let set_output_module_implementation_name fname =
  if !output_module_implementation_name = "-"
  then output_module_implementation_name := fname
  else failwith "Output module implementation name is already set.";;

let output_module_interface_name = ref "-";;
let get_output_module_interface_name () = !output_module_interface_name;;
let set_output_module_interface_name fname =
  if !output_module_interface_name = "-"
  then output_module_interface_name := fname
  else failwith "Output module interface name is already set.";;

(* Parsing of the current mlm file and generation of the corresponding
   interface and implementation files. *)

let generate_files fname =
  let ic =
    match fname with
    | "-" -> stdin
    | s -> open_in s in
  let ib = Lexing.from_channel ic in
  ib.Lexing.lex_curr_p <-
    { ib.Lexing.lex_curr_p with Lexing.pos_fname = fname }; 
  let sis = Parse.implementation ib in
  let ob = Buffer.create 1024 in
  try
    let sharing = get_sharable_target () in
    if sharing then print_sharing_preamble ob;
    List.iter (check_and_genr sharing ob) sis;
    print_endline (Buffer.contents ob)
  with
  | Mod_sigitem_not_allowed ->
      prerr_endline "Only type definitions are allowed."
  | e -> Check.pr_error e;;

(* Parsing the command line. *)

let get_usage () =
  "Usage: mocac <options> <.mlm file>";;

let print_moca_version v =
  prerr_endline
   (Printf.sprintf
      "The Moca relational types compiler, version %s" v);
  exit 0;;

let print_moca_short_version () =
  print_moca_version
    (Printf.sprintf "%.2f" Configuration.moca_version_number);;

let print_moca_full_version () =
  print_moca_version Configuration.moca_full_version;;

let set_moca_to_verbose () = Configuration.set_verbose true;;

(* The main procedure *)

let main () =
  Arg.parse [
    ("--sharing", Arg.Unit set_sharable_target,
     " generate maximally sharing data structures.");
    ("--verbose", Arg.Unit set_moca_to_verbose,
     " be verbose during generation.");
    ("-v", Arg.Unit print_moca_short_version,
     " print the moca version.");
    ("--version", Arg.Unit print_moca_full_version,
     " print the full moca version, sub-version and release date.");
    ("-c", Arg.String set_input_file_name,
     " generate a module for the input file argument.");
(*
    ("-o", Arg.String set_output_module_name,
     " output the generated module with name the file name argument.");
    ("-oml", Arg.String set_output_module_implementation_name,
     " output the generated module implementation in the file argument.");
    ("-omli", Arg.String set_output_module_interface_name,
     " output the generated module interface in the file argument.");
*)
  ]
    set_input_file_name
    (get_usage ());
  generate_files (get_input_file_name ());
  exit 0;;

(* Run the main procedure. *)

main ();;
